/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SAMPLINGPOINT_H
#define DPC_IMPL_TYPE_SAMPLINGPOINT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_boolean.h"

namespace dpc {
struct SamplingPoint {
    ::Float x{ 0.0F };
    ::Float y{ 0.0F };
    ::Float curve{ 0.0F };
    ::Boolean valid{ false };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(x);
        fun(y);
        fun(curve);
        fun(valid);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(x);
        fun(y);
        fun(curve);
        fun(valid);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("x", x);
        fun("y", y);
        fun("curve", curve);
        fun("valid", valid);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("x", x);
        fun("y", y);
        fun("curve", curve);
        fun("valid", valid);
    }

    friend bool operator==(const ::dpc::SamplingPoint& lhs, const ::dpc::SamplingPoint& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.x - rhs.x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.y - rhs.y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.curve - rhs.curve)) < DBL_EPSILON) && (lhs.valid == rhs.valid);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SAMPLINGPOINT_H
